"use strict";
/*!
 * node-rcon
 * Copyright(c) 2012 Justin Li <j-li.net>
 * MIT Licensed
 */
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = __importDefault(require("events"));
var net_1 = __importDefault(require("net"));
var dgram_1 = __importDefault(require("dgram"));
var buffer_1 = require("buffer");
var PacketType = {
    COMMAND: 0x02,
    AUTH: 0x03,
    RESPONSE_VALUE: 0x00,
    RESPONSE_AUTH: 0x02
};
var Rcon = /** @class */ (function (_super) {
    __extends(Rcon, _super);
    function Rcon(host, port, password, options) {
        var _this = _super.call(this) || this;
        _this.send = function (data, cmd, id) {
            var sendBuf;
            if (_this.tcp) {
                cmd = cmd || PacketType.COMMAND;
                id = id || _this.rconId;
                var length_1 = buffer_1.Buffer.byteLength(data);
                sendBuf = buffer_1.Buffer.alloc(length_1 + 14);
                sendBuf.writeInt32LE(length_1 + 10, 0);
                sendBuf.writeInt32LE(id, 4);
                sendBuf.writeInt32LE(cmd, 8);
                sendBuf.write(data, 12);
                sendBuf.writeInt16LE(0, length_1 + 12);
            }
            else {
                if (_this.challenge && !_this._challengeToken) {
                    _this.emit('error', new Error('Not authenticated'));
                    return;
                }
                var str = "rcon ";
                if (_this._challengeToken)
                    str += _this._challengeToken + " ";
                if (_this.password)
                    str += _this.password + " ";
                str += data + "\n";
                sendBuf = buffer_1.Buffer.alloc(4 + buffer_1.Buffer.byteLength(str));
                sendBuf.writeInt32LE(-1, 0);
                sendBuf.write(str, 4);
            }
            _this._sendSocket(sendBuf);
        };
        _this._sendSocket = function (buf) {
            if (_this._tcpSocket) {
                _this._tcpSocket.write(buf.toString('binary'), 'binary');
            }
            else if (_this._udpSocket) {
                _this._udpSocket.send(buf, 0, buf.length, _this.port, _this.host);
            }
        };
        _this.connect = function () {
            var self = _this;
            if (_this.tcp) {
                _this._tcpSocket = net_1.default.createConnection(_this.port, _this.host);
                _this._tcpSocket.on('data', function (data) { self._tcpSocketOnData(data); })
                    .on('connect', function () { self.socketOnConnect(); })
                    .on('error', function (err) { self.emit('error', err); })
                    .on('end', function () { self.socketOnEnd(); });
            }
            else {
                _this._udpSocket = dgram_1.default.createSocket("udp4");
                _this._udpSocket.on('message', function (data) { self._udpSocketOnData(data); })
                    .on('listening', function () { self.socketOnConnect(); })
                    .on('error', function (err) { self.emit('error', err); })
                    .on('close', function () { self.socketOnEnd(); });
                _this._udpSocket.bind(0);
            }
        };
        _this.disconnect = function () {
            if (_this._tcpSocket)
                _this._tcpSocket.end();
            if (_this._udpSocket)
                _this._udpSocket.close();
        };
        _this.setTimeout = function (timeout, callback) {
            if (!_this._tcpSocket)
                return;
            var self = _this;
            _this._tcpSocket.setTimeout(timeout, function () {
                self._tcpSocket.end();
                if (callback)
                    callback();
            });
        };
        _this._udpSocketOnData = function (data) {
            var a = data.readUInt32LE(0);
            if (a == 0xffffffff) {
                var str = data.toString("utf-8", 4);
                var tokens = str.split(" ");
                if (tokens.length == 3 && tokens[0] == "challenge" && tokens[1] == "rcon") {
                    _this._challengeToken = tokens[2].substr(0, tokens[2].length - 1).trim();
                    _this.hasAuthed = true;
                    _this.emit('auth');
                }
                else {
                    _this.emit('response', str.substr(1, str.length - 2));
                }
            }
            else {
                _this.emit('error', new Error("Received malformed packet"));
            }
        };
        _this._tcpSocketOnData = function (data) {
            if (_this.outstandingData != null) {
                data = buffer_1.Buffer.concat([_this.outstandingData, data], _this.outstandingData.length + data.length);
                _this.outstandingData = null;
            }
            while (data.length) {
                var len = data.readInt32LE(0);
                if (!len)
                    return;
                var id = data.readInt32LE(4);
                var type = data.readInt32LE(8);
                if (len >= 10 && data.length >= len + 4) {
                    if (id == _this.rconId) {
                        if (!_this.hasAuthed && type == PacketType.RESPONSE_AUTH) {
                            _this.hasAuthed = true;
                            _this.emit('auth');
                        }
                        else if (type == PacketType.RESPONSE_VALUE) {
                            // Read just the body of the packet (truncate the last null byte)
                            // See https://developer.valvesoftware.com/wiki/Source_RCON_Protocol for details
                            var str = data.toString('utf8', 12, 12 + len - 10);
                            if (str.charAt(str.length - 1) === '\n') {
                                // Emit the response without the newline.
                                str = str.substring(0, str.length - 1);
                            }
                            _this.emit('response', str);
                        }
                    }
                    else {
                        _this.emit('error', new Error("Authentication failed"));
                    }
                    data = data.slice(12 + len - 8);
                }
                else {
                    // Keep a reference to the chunk if it doesn't represent a full packet
                    _this.outstandingData = data;
                    break;
                }
            }
        };
        _this.socketOnConnect = function () {
            _this.emit('connect');
            if (_this.tcp) {
                _this.send(_this.password, PacketType.AUTH);
            }
            else if (_this.challenge) {
                var str = "challenge rcon\n";
                var sendBuf = buffer_1.Buffer.alloc(str.length + 4);
                sendBuf.writeInt32LE(-1, 0);
                sendBuf.write(str, 4);
                _this._sendSocket(sendBuf);
            }
            else {
                var sendBuf = buffer_1.Buffer.alloc(5);
                sendBuf.writeInt32LE(-1, 0);
                sendBuf.writeUInt8(0, 4);
                _this._sendSocket(sendBuf);
                _this.hasAuthed = true;
                _this.emit('auth');
            }
        };
        _this.socketOnEnd = function () {
            _this.emit('end');
            _this.hasAuthed = false;
        };
        options = options || {};
        _this.host = host;
        _this.port = port;
        _this.password = password;
        _this.rconId = options.id || 0x0012D4A6; // This is arbitrary in most cases
        _this.hasAuthed = false;
        _this.outstandingData = null;
        _this.tcp = options.tcp == null ? true : options.tcp;
        _this.challenge = options.challenge == null ? true : options.challenge;
        _this._challengeToken = "";
        events_1.default.EventEmitter.call(_this);
        return _this;
    }
    ;
    return Rcon;
}(events_1.default.EventEmitter));
exports.Rcon = Rcon;
exports.default = Rcon;
